/*******************************************************************************
* (c) Copyright 2014 Biricha Digital Power Limited
* AUTHOR        : Biricha Digital Power Ltd
* CREATION DATE : 03/07/2014
* COPYRIGHT     : Copyright Biricha Digital Power Limited 2014. All rights
*                 reserved. Reproduction in whole or part is prohibited without
*                 written consent of the copyright owner. You may receive a free
*                 commercial license by contacting Biricha Digital. Do not
*                 remove or change this copyright notice.
*
*******************************************************************************/

#include "bdp.h"
#include "smps_control.h"

extern void SMPS_Controller3P3ZInitialize(SMPS_3P3Z_T *controllerData );
extern void SMPS_Controller2P2ZInitialize(SMPS_2P2Z_T *controllerData );

void CTRL_3p3z_iq_pre_init( CTRL_3p3z_iq_pre_data *pCtrl,
    uint16_t    Ref, uint16_t Kerr, int16_t KcShift, int16_t Kpwm,
    uint16_t  cB0,uint16_t  cB1,uint16_t  cB2,uint16_t  cB3,
    uint16_t  cA1,uint16_t  cA2,uint16_t  cA3,
    volatile uint16_t *pFeedBack, volatile uint16_t *pOut
)
{
    pCtrl->xData->m_pFeedBack      = pFeedBack;
    pCtrl->xData->m_pAccal1        = &ACCAL;
    pCtrl->xData->m_KcShift        = KcShift;
    pCtrl->xData->m_KpwmPost       = Kpwm;
    pCtrl->xData->m_B3             = cB3;
    pCtrl->xData->m_B2             = cB2;
    pCtrl->xData->m_B1             = cB1;
    pCtrl->xData->m_A3             = cA3;
    pCtrl->xData->m_A2             = cA2;
    pCtrl->xData->m_A1             = cA1;
    pCtrl->xData->m_pAccal2        = &ACCAL;

    pCtrl->yData->m_Ref            = Ref;
    pCtrl->yData->m_Kerr           = Kerr;
    pCtrl->yData->m_B0             = cB0;
    pCtrl->yData->m_Accal          = 0;
    pCtrl->yData->m_Accah          = 0;
    pCtrl->yData->m_Accau          = 0;
    pCtrl->yData->m_pOut           = pOut;
    pCtrl->yData->m_E2             = 0;
    pCtrl->yData->m_E1             = 0;
    pCtrl->yData->m_E0             = 0;
    pCtrl->yData->m_U2             = 0;
    pCtrl->yData->m_U1             = 0;
    pCtrl->yData->m_U0             = 0;
}


void CTRL_3p3z_init( SMPS_3P3Z_T *pCtrl,
                     int16_t *pCntrlACoefficients,
                     int16_t *pCntrlBCoefficients,
                     int16_t *pCntrlControlHistory,
                     int16_t *pCntrlErrorHistory,
                     uint16_t  cB0,uint16_t  cB1,uint16_t  cB2,uint16_t  cB3,
                     uint16_t  cA1,uint16_t  cA2,uint16_t  cA3,
                     uint16_t pre_shift,
                     int16_t post_shift,
                     int16_t K,
                     uint16_t maxOutput,
                     uint16_t minOutput
)
{
    pCtrl->aCoefficients = pCntrlACoefficients;
    pCtrl->bCoefficients = pCntrlBCoefficients;

    pCtrl->controlHistory = pCntrlControlHistory;
    pCtrl->errorHistory = pCntrlErrorHistory;

    SMPS_Controller3P3ZInitialize(pCtrl);

    pCtrl->aCoefficients[0] = cA1;
    pCtrl->aCoefficients[1] = cA2;
    pCtrl->aCoefficients[2] = cA3;

    pCtrl->bCoefficients[0] = cB0;
    pCtrl->bCoefficients[1] = cB1;
    pCtrl->bCoefficients[2] = cB2;
    pCtrl->bCoefficients[3] = cB3;

    pCtrl->preShift = pre_shift;
    pCtrl->postShift = -post_shift; // Note: Left shift is represented by
                                    // a negative number for this
                                    // controller
    pCtrl->postScaler = K;

    pCtrl->maxOutput = maxOutput;
    pCtrl->minOutput = minOutput;
}


void CTRL_2p2z_init( SMPS_2P2Z_T *pCtrl,
                     int16_t *pCntrlACoefficients,
                     int16_t *pCntrlBCoefficients,
                     int16_t *pCntrlControlHistory,
                     int16_t *pCntrlErrorHistory,
                     uint16_t  cB0,uint16_t  cB1,uint16_t  cB2,
                     uint16_t  cA1,uint16_t  cA2,
                     uint16_t pre_shift,
                     int16_t post_shift,
                     int16_t K,
                     uint16_t maxOutput,
                     uint16_t minOutput
)
{
    pCtrl->aCoefficients = pCntrlACoefficients;
    pCtrl->bCoefficients = pCntrlBCoefficients;

    pCtrl->controlHistory = pCntrlControlHistory;
    pCtrl->errorHistory = pCntrlErrorHistory;

    SMPS_Controller2P2ZInitialize(pCtrl);

    pCtrl->aCoefficients[0] = cA1;
    pCtrl->aCoefficients[1] = cA2;

    pCtrl->bCoefficients[0] = cB0;
    pCtrl->bCoefficients[1] = cB1;
    pCtrl->bCoefficients[2] = cB2;

    pCtrl->preShift = pre_shift;
    pCtrl->postShift = -post_shift; // Note: Left shift is represented by
                                    // a negative number for this
                                    // controller
    pCtrl->postScaler = K;

    pCtrl->maxOutput = maxOutput;
    pCtrl->minOutput = minOutput;
}

void CTRL_SoftStartStop_init( CTRL_SoftStartStop *pInfo,
                    uint16_t Max,
                    uint16_t Count,
                    int16_t Step,
                    CTRL_SS_Mode Dir
)
{
    pInfo->m_Ref = 0; // Initially zero
    pInfo->m_Max = Max;
    pInfo->m_Count = Count;
    pInfo->m_Step = Step;
    pInfo->m_Dir = Dir;
}

void CTRL_SoftStartStop_Update( CTRL_SoftStartStop *pInfo )
{
    uint16_t count_val = 0;

    if (pInfo->m_Dir)
    {
        // Increment reference
        if (pInfo->m_Ref < pInfo->m_Max)
        {
            // Soft Starting
            pInfo->m_Ref = pInfo->m_Ref + pInfo->m_Step;
        } else {
            pInfo->m_Ref = pInfo->m_Max;
        }
    } else {
        // Decrement reference
        if (pInfo->m_Ref > pInfo->m_Step)
        {
            pInfo->m_Ref = pInfo->m_Ref - pInfo->m_Step;
        } else {
            // Soft Stopped
            pInfo->m_Ref = 0; // Set reference to zero
        }
    }

    while (count_val < SS_COUNT)
    {
        count_val++;
    }
}
