/*******************************************************************************
* (c) Copyright 2014 Biricha Digital Power Limited
* AUTHOR        : Biricha Digital Power Ltd
* CREATION DATE : 03/07/2014
* COPYRIGHT     : Copyright Biricha Digital Power Limited 2014. All rights
*                 reserved. Reproduction in whole or part is prohibited without
*                 written consent of the copyright owner. You may receive a free
*                 commercial license by contacting Biricha Digital. Do not
*                 remove or change this copyright notice.
*
*******************************************************************************/

#ifndef CSL_CTRL_H
#define CSL_CTRL_H
#include <stdint.h>
#include <xc.h>

#define SS_COUNT (1000LL)*(FCY)/(8LL*1000000LL) // Counter that gives us approx 1ms delay

typedef int16_t _iq15;

typedef struct CTRL_3p3z_iq_pre_xdata
{
    //            w8
    volatile uint16_t  *m_pFeedBack;   //C_ADC_PTR        0
    volatile uint16_t  *m_pAccal1;     //C_ACCAL_PTR1     1
    int16_t             m_KcShift;      //C_KC_SHIFT      2
    int16_t             m_KpwmPost;     //C_KPWM_POST     3
    _iq15               m_B3;           //C_B3            4
    _iq15               m_B2;           //C_B2            5
    _iq15               m_B1;           //C_B1            6
    _iq15               m_A3;           //C_A3            7
    _iq15               m_A2;           //C_A2            8
    _iq15               m_A1;           //C_A1            9
    volatile uint16_t  *m_pAccal2;     //C_ACCAL_PTR2    10
}CTRL_3p3z_iq_pre_xdata;


typedef struct CTRL_3p3z_iq_pre_ydata
{
    //            w10
    int16_t             m_Ref;          //V_REF           0
    uint16_t            m_Kerr;         //V_KERR          1
    _iq15               m_B0;           //V_B0            2
    uint16_t            m_Accal;        //V_ACCAL         3
    uint16_t            m_Accah;        //V_ACCAH         4
    uint16_t            m_Accau;        //V_ACCAU         5
    volatile uint16_t  *m_pOut;         //V_OUT_PTR       6
    _iq15               m_E2;           //V_E2            7
    _iq15               m_E1;           //V_E1            8
    _iq15               m_E0;           //V_E0            9
    _iq15               m_U2;           //V_U2            10
    _iq15               m_U1;           //V_U1            11
    _iq15               m_U0;           //V_U0            12
} CTRL_3p3z_iq_pre_ydata;

typedef struct CTRL_3p3z_iq_pre_data
{
    CTRL_3p3z_iq_pre_xdata  *xData;
    CTRL_3p3z_iq_pre_ydata  *yData;
}CTRL_3p3z_iq_pre_data;

typedef struct CTRL_SoftStartStop
{
    volatile int16_t    m_Ref;         // 0
    uint16_t            m_Max;         // 1
    uint16_t            m_Count;       // 2
    int16_t             m_Step;        // 3
    uint16_t            m_Dir;         // 4
} CTRL_SoftStartStop;

typedef enum CTRL_SS_Mode
{
    SS_Mode_Stopping = 0,
    SS_Mode_Starting = 1
}CTRL_SS_Mode;



#define CTRL_3p3z_iq_pre( pCtrl ) inline_CTRL_3p3z_iq_pre_func( (pCtrl)->xData, (pCtrl)->yData)
extern void CTRL_3p3z_iq_pre_func(CTRL_3p3z_iq_pre_xdata*, CTRL_3p3z_iq_pre_ydata*);

extern void CTRL_3p3z_iq_pre_init( CTRL_3p3z_iq_pre_data *pCtrl,
    uint16_t    Ref, uint16_t Kerr, int16_t KcShift, int16_t Kpwm,
    uint16_t  cB0,uint16_t  cB1,uint16_t  cB2,uint16_t  cB3,
    uint16_t  cA1,uint16_t  cA2,uint16_t  cA3,
    volatile uint16_t *pFeedBack, volatile uint16_t *pOut
);


extern void CTRL_3p3z_init( SMPS_3P3Z_T *pCtrl,
                     int16_t *pCntrlACoefficients,
                     int16_t *pCntrlBCoefficients,
                     int16_t *pCntrlControlHistory,
                     int16_t *pCntrlErrorHistory,
                     uint16_t  cB0,uint16_t  cB1,uint16_t  cB2,uint16_t  cB3,
                     uint16_t  cA1,uint16_t  cA2,uint16_t  cA3,
                     uint16_t pre_shift,
                     int16_t post_shift,
                     int16_t K,
                     uint16_t maxOutput,
                     uint16_t minOutput
);

extern void CTRL_2p2z_init( SMPS_2P2Z_T *pCtrl,
                     int16_t *pCntrlACoefficients,
                     int16_t *pCntrlBCoefficients,
                     int16_t *pCntrlControlHistory,
                     int16_t *pCntrlErrorHistory,
                     uint16_t  cB0,uint16_t  cB1,uint16_t  cB2,
                     uint16_t  cA1,uint16_t  cA2,
                     uint16_t pre_shift,
                     int16_t post_shift,
                     int16_t K,
                     uint16_t maxOutput,
                     uint16_t minOutput
);

extern void CTRL_SoftStartStop_init( CTRL_SoftStartStop *pInfo,
                    uint16_t Max,
                    uint16_t Count,
                    int16_t Step,
                    CTRL_SS_Mode Dir
);

extern void CTRL_SoftStartStop_Update( CTRL_SoftStartStop *pInfo );


        //Operational modes for fractional computation:
        #define SATA_ON (1<<7)                    // AccA sat. enabled
        #define SATB_ON (1<<6)                    // AccB sat. enabled
        #define SATDW_ON (1<<5)                   // data write sat. enabled
        #define ACCSAT_ON (1<<4)                  // Accs sat. 9.31 enabled
        #define FRACT_SET (SATA_ON|SATB_ON|SATDW_ON|ACCSAT_ON)    // set mask

        #define RND_OFF (~(1<<1))                 // convergent rnd. enabled
        #define IF_OFF (~(1<<0))                  // multiply fractional enabled
        #define FRACT_RESET (RND_OFF&IF_OFF)      // reset mask

        #define FRACT_MODE (FRACT_SET&FRACT_RESET)  // fractional mask


#define inline_CTRL_3p3z_iq_pre( pCtrl ) inline_CTRL_3p3z_iq_pre_func( (pCtrl)->xData, (pCtrl)->yData)
__inline static void inline_CTRL_3p3z_iq_pre_func( CTRL_3p3z_iq_pre_xdata *xData, CTRL_3p3z_iq_pre_ydata* yData )
{
    asm( "       "
"\n     ; Save working registers."
"\n     push    CORCON                  ; Prepare CORCON for fractional computation."
"\n"
"\n     mov     #%[iFRACT_MODE],w4          ;set fract mode"
"\n     mov     w4,CORCON"
"\n"
"\n     mov %[XDATA], w8"
"\n     mov %[YDATA], w10"
"\n"
"\n     ;E0 = K_ERR*(REF - *ADC_PTR);"
"\n     clr    a, [w8]+=2,w4, [w10]+=2,w5         ; w4   = &ADC_PTR, w5=REF"
"\n     sub    w5,[w4], w4                        ; w4   = REF - *ADC_PTR"
"\n     mul.ss w4,[w10++], w6                     ; w7w6 = K_ERR( REF - *&ADC_PTR )"
"\n     mov    w6,[w10+#2*7]                      ; store E0"
"\n"
"\n     clr    a, [w8]+=2,w5, [w10]+=2,w4         ; w4=#ACCAL_PTR1, w4=B0"
"\n     ;      src     dst"
"\n     mov    [w10++],[w5++]   ;ACCAL"
"\n     mov    [w10++],[w5++]   ;ACCAH"
"\n     mov    [w10++],[w5++]   ;ACCAU"
"\n"
"\n     mac    w6*w4,a, [w8]+=2,w4                ; acc += B0*E0, w4 = KC_SHIFT"
"\n     sftac  a,w4                               ; acc << KC_SHIFT"
"\n     movsac b, [w8]+=2, w4, [w10]+=2, w5, w13  ; w13 = sat( acc ), w4=K_PWM_POST, w5=&OUT_PTR"
"\n     mov    w13,[w10+#2*5]                     ; store U0"
"\n"
"\n     mul.ss w13,w4,w6                          ; w7w6 = U0 x K_PWM_POST"
"\n     btsc   w7,#15                             ;if ( w7 < 0 )"
"\n     clr    w7                                 ;   w7 = 0"
"\n     mov    w7,[w5]                            ; *OUT_PTR = w7"
"\n     ;BCLR LATB, #6       ;set ADC gpio pin to zero - for timing only"
"\n                                               ;                   post fetch"
"\n     clr    a, [w8]+=2, w4, [w10]+=2, w5            ;                   w4=B3 w5=E2"
"\n     mac    w4*w5, a, [w8]+=2, w4, [w10]+=2, w5     ;acc += B3*E2       w4=B2 w5=E1"
"\n     mov    w5,[w10-#2*2]                           ;  E2 = vE1"
"\n     mac    w4*w5, a, [w8]+=2, w4, [w10]+=2, w5     ;acc += B2*E1       w4=B1 w5=E0"
"\n     mov    w5,[w10-#2*2]                           ;  E1 = E0"
"\n     mac    w4*w5, a, [w8]+=2, w4, [w10]+=2, w5     ;acc += B1* E0      w4=A3 w5=U2"
"\n"
"\n     mac    w4*w5, a, [w8]+=2, w4, [w10]+=2, w5     ;acc += A3*U2       w4=A2 w5=U1"
"\n     mov    w5,[w10-#2*2]                           ;  U2 = U1"
"\n     mac    w4*w5, a, [w8]+=2, w4, [w10]+=2, w5     ;acc += A2*U1       w4=A1 w5=U0"
"\n     mov    w5,[w10-#2*2]                           ;  U1 = U0"
"\n     mac    w4*w5, a, [w8],    w4                   ;acc += A1*U0       w4=ACCAL_PTR2 (duplicate)"
"\n"
"\n     ;save acc"
"\n     sub   #2*10, w10"
"\n     ;      src     dst"
"\n     mov   [w4++],[w10++]   ;ACCAL *w10 = *w4"
"\n     mov   [w4++],[w10++]   ;ACCAH"
"\n     mov   [w4++],[w10++]   ;ACCAU"
"\n"
"\n     pop   CORCON                  ; restore CORCON."
        :
        : [iFRACT_MODE]"i"(FRACT_MODE), [XDATA]"g"(xData), [YDATA]"g"(yData)
        : "w4", "w5", "w6", "w7","w8","w10"
        );
}

#endif
