;*******************************************************************************
; (c) Copyright 2014 Biricha Digital Power Limited
; AUTHOR        : Biricha Digital Power Ltd
; CREATION DATE : 03/07/2014
; COPYRIGHT     : Copyright Biricha Digital Power Limited 2014. All rights
;                 reserved. Reproduction in whole or part is prohibited without
;                 written consent of the copyright owner. You may receive a free
;                 commercial license by contacting Biricha Digital. Do not
;                 remove or change this copyright notice.
;
;*******************************************************************************

        .global _CTRL_3p3z_iq_pre_func          ; provide global scope to routine

        ;.include "3p3z.h"

        ; Operational modes for fractional computation:
        .equ    SATA_ON,1<<7                    ; AccA sat. enabled
        .equ    SATB_ON,1<<6                    ; AccB sat. enabled
        .equ    SATDW_ON,1<<5                   ; data write sat. enabled
        .equ    ACCSAT_ON,1<<4                  ; Accs sat. 9.31 enabled
        .equ    FRACT_SET,SATA_ON|SATB_ON|SATDW_ON|ACCSAT_ON    ; set mask

        .equ    RND_OFF,~(1<<1)                 ; convergent rnd. enabled
        .equ    IF_OFF,~(1<<0)                  ; multiply fractional enabled
        .equ    FRACT_RESET,RND_OFF&IF_OFF      ; reset mask

        .equ    FRACT_MODE,FRACT_SET&FRACT_RESET        ; fractional mask

_CTRL_3p3z_iq_pre_func:
        ;btg LATD, #1
        ; Save working registers.
        push.s
        push    w4
        push    w5
        push    w6
        push    w7
        push    w8
        push    w10
        push    CORCON                  ; Prepare CORCON for fractional computation.

        mov     #FRACT_MODE,w4          ;set fract mode
        mov     w4,CORCON

        mov W0, w8
        mov W1, w10

        ;E0 = K_ERR*(REF - *ADC_PTR);
        clr    a, [w8]+=2,w4, [w10]+=2,w5         ; w4   = &ADC_PTR, w5=REF
        sub    w5,[w4], w4                        ; w4   = REF - *ADC_PTR
        mul.ss w4,[w10++], w6                     ; w7w6 = K_ERR( REF - *&ADC_PTR )
        mov    w6,[w10+#2*7]                      ; store E0

        clr    a, [w8]+=2,w5, [w10]+=2,w4         ; w4=#ACCAL_PTR1, w4=B0
        ;      src     dst
        mov    [w10++],[w5++]   ;ACCAL
        mov    [w10++],[w5++]   ;ACCAH
        mov    [w10++],[w5++]   ;ACCAU

        mac    w6*w4,a, [w8]+=2,w4                ; acc += B0*E0, w4 = KC_SHIFT
        sftac  a,w4                               ; acc << KC_SHIFT
        movsac b, [w8]+=2, w4, [w10]+=2, w5, w13  ; w13 = sat( acc ), w4=K_PWM_POST, w5=&OUT_PTR
        mov    w13,[w10+#2*5]                     ; store U0

        mul.ss w13,w4,w6                          ; w7w6 = U0 x K_PWM_POST
        btsc   w7,#15                             ;if ( w7 < 0 )
        clr    w7                                 ;   w7 = 0
        mov    w7,[w5]                            ; *OUT_PTR = w7

        ;BCLR LATB, #6       ;set ADC gpio pin to zero - for timing only
                                                       ;                   post fetch
        clr    a, [w8]+=2, w4, [w10]+=2, w5            ;                   w4=B3 w5=E2
        mac    w4*w5, a, [w8]+=2, w4, [w10]+=2, w5     ;acc += B3*E2       w4=B2 w5=E1
        mov    w5,[w10-#2*2]                           ;  E2 = vE1
        mac    w4*w5, a, [w8]+=2, w4, [w10]+=2, w5     ;acc += B2*E1       w4=B1 w5=E0
        mov    w5,[w10-#2*2]                           ;  E1 = E0
        mac    w4*w5, a, [w8]+=2, w4, [w10]+=2, w5     ;acc += B1* E0      w4=A3 w5=U2

        mac    w4*w5, a, [w8]+=2, w4, [w10]+=2, w5     ;acc += A3*U2       w4=A2 w5=U1
        mov    w5,[w10-#2*2]                           ;  U2 = U1
        mac    w4*w5, a, [w8]+=2, w4, [w10]+=2, w5     ;acc += A2*U1       w4=A1 w5=U0
        mov    w5,[w10-#2*2]                           ;  U1 = U0
        mac    w4*w5, a, [w8],    w4                   ;acc += A1*U0       w4=ACCAL_PTR2 (duplicate)

        ;save acc
        sub   #2*10, w10
        ;      src     dst
        mov   [w4++],[w10++]   ;ACCAL *w10 = *w4
        mov   [w4++],[w10++]   ;ACCAH
        mov   [w4++],[w10++]   ;ACCAU

        pop   CORCON                  ; restore CORCON.
        pop   w10                     ; Restore working registers.
        pop   w8
        pop   w7
        pop   w6
        pop   w5
        pop   w4
        pop.s

        return


