/*******************************************************************************
* (c) Copyright 2014 Biricha Digital Power Limited
* AUTHOR        : Biricha Digital Power Ltd
* CREATION DATE : 03/07/2014
* COPYRIGHT     : Copyright Biricha Digital Power Limited 2014. All rights
*                 reserved. Reproduction in whole or part is prohibited without
*                 written consent of the copyright owner. You may receive a free
*                 commercial license by contacting Biricha Digital. Do not
*                 remove or change this copyright notice.
*
*******************************************************************************/

#include "bdp.h"

void PWM_init(uint16_t PeriodTicks)
{
    // set the div for hi-res or normal
    PTCON2bits.PCLKDIV  = PWM_PCLKDIV;
    PTPER               = PeriodTicks;	// Set Frequency
    IOCON1bits.PENH = 0;    // PWM1H pin is controlled by GPIO module
    IOCON1bits.PENL = 0;    // PWM1L pin is controlled by GPIO module
    IOCON2bits.PENH = 0;    // PWM2H pin is controlled by GPIO module
    IOCON2bits.PENL = 0;    // PWM2L pin is controlled by GPIO module
}

void PWM_enable( bool Enable)

{
    PTCONbits.PTEN      = Enable;            // Enable the PWM
}

void PWM1_config( void )
{
    IOCON1bits.PENH = 0;          // PWM1H is controlled by I/O module
    IOCON1bits.PENL = 0;          // PWM1L is controlled by I/O module

    IOCON1bits.PMOD = PWM_Mode_Complementary;          // Complementary Mode

    IOCON1bits.POLH = 0;          // Drive signals are active-high
    IOCON1bits.POLL = 0;          // Drive signals are active-high

    IOCON1bits.OVRENH = 0;	  // Disable Override feature for shutdown PWM
    IOCON1bits.OVRENL = 0;	  // Disable Override feature for shutdown PWM
    IOCON1bits.OVRDAT = 0b00;	  // Shut down PWM with Over ride 0 on PWMH and PWML
    IOCON1bits.FLTDAT = 0b00;	  // Shut down PWM with Over ride 0 on PWMH and PWML

    PWMCON1bits.DTC     = 2;        // Deadtime disabled
    PWMCON1bits.IUE     = 0;        // Disable Immediate duty cycle updates
    PWMCON1bits.ITB     = 0;        // Select Primary Timebase mode
    PWMCON1bits.TRGIEN  = 0;        // trigger event triggers int
    FCLCON1bits.FLTMOD  = 3;        // fault disabled
    FCLCON1bits.CLMOD   = PWM_Cur_Limit_Mode_Disabled;
    PWMCON1bits.MDCS    = 0;         //PDCx/SDCx register provides duty cycle


    TRGCON1bits.TRGDIV = 0;       // Trigger interrupt generated once every PWM cycle
    TRGCON1bits.TRGSTRT = 0;      // Trigger generated after waiting 0 PWM cycles

    PWM1_setTrigger(0);           // Set Initial Trigger location
    PWM1_setDutyH( 0 );         // Set duty to 0
    PWM1_setDutyL( 0 );         // Set duty to 0
}

void PWM1_deadtime( PWM_Deadband Mode, uint16_t Ticks )
{
    PWMCON1bits.DTC     = Mode;        // Deadtime disabled
    DTR1 = Ticks;
    ALTDTR1 = Ticks;
}
void PWM1_setOutput( PWM_Mode Mode, PWM_Pin_Output PwmH, PWM_Pin_Output PwmL)
{
    //return;
    IOCON1bits.POLH = PwmH == PWM_Pin_Active_Low;  // Drive signals are active-high
    IOCON1bits.PENH = PwmH != PWM_Pin_Disable;  // PWM1H is controlled by PWM module

    IOCON1bits.POLL = PwmL == PWM_Pin_Active_Low;  // Drive signals are active-high
    IOCON1bits.PENL = PwmL != PWM_Pin_Disable;  // PWM1H is controlled by PWM module

    IOCON1bits.PMOD = Mode;
}

void PWM1_blanking( PWM_Current_Limit_Mode CurrentLimitMode, uint16_t Ticks, PWM_Pin_Level PwmH, PWM_Pin_Level PwmL )
{
    // In order to block the sensed current spike at the
    // turn on edge of the HS FET, use Leading Edge blanking.
    LEBCON1bits.LEB      = Ticks;	  // 8.32n Steps x PWM_DIV

    FCLCON1bits.IFLTMOD = 0; // Independent Fault Mode Enable bit disabled

    FCLCON1bits.FLTSRC  = 0; // Fault input source is Analog Comparator 1
    FCLCON1bits.FLTPOL  = 0; // Fault Input source is active-high

    FCLCON1bits.CLSRC  = 0; // Current limit input source is Analog Comparator 1
    FCLCON1bits.CLPOL  = 0; // Current limit Input source is active-high

    FCLCON1bits.CLMOD  = CurrentLimitMode; // Enable current-limit function
    FCLCON1bits.FLTMOD = 3; // 1 = Enable Cycle-by-Cycle Fault mode 3= disable

    IOCON1bits.FLTDAT = 0; // PWMxH and PWMxL are driven inactive on occurrence of fault
    IOCON1bits.CLDAT  = (PwmH==PWM_Pin_Low? 0:2) | (PwmL==PWM_Pin_Low? 0:1); // PWMxH and PWMxL are driven inactive on occurrence of fault
    IOCON1bits.OVRDAT = IOCON1bits.CLDAT;

    LEBCON1bits.PHR = 1; // Rising edge of PWMxH will trigger LEB counter
    LEBCON1bits.PHF = 0; // Falling edge of PWMxH is ignored by LEB counter
    LEBCON1bits.PLR = 0; // Rising edge of PWMxL will trigger LEB counter
    LEBCON1bits.PLF = 0; // Falling edge of PWMxL is ignored by LEB counter

    LEBCON1bits.FLTLEBEN = 0; // Leading-Edge Blanking is applied to selected fault input
    LEBCON1bits.CLLEBEN  = 1; // Leading-Edge Blanking is applied to selected current-limit input

    PWMCON1bits.XPRES  = 0; // External pins do not affect PWM time base reset
    PWMCON1bits.FLTIEN = 0; // Enable fault interrupt
    PWMCON1bits.CLIEN  = 1; // Enable current-limit interrupt - we need this to enable PWMCAP1
    //when we enable CLIEN PWMCAP1 starts to work, but we also get a double inetrrupt when
    // we enable PWM interrupts, so you will need to check CLSTAT
}

void PWM1_interrupt( INT_Priority Priority  )
{
    PWMCON1bits.TRGIEN = Priority != INT_Priority_Disabled;
    // Enable the PWM interrupts
    IFS5bits.PWM1IF=0;			// Interrupt flag status register
    IPC23bits.PWM1IP = Priority;               // Set PWM interrupt priority
    IEC5bits.PWM1IE=1;			// Interrupt request is enabled
}