/*******************************************************************************
* (c) Copyright 2014 Biricha Digital Power Limited
* AUTHOR        : Biricha Digital Power Ltd
* CREATION DATE : 03/07/2014
* COPYRIGHT     : Copyright Biricha Digital Power Limited 2014. All rights
*                 reserved. Reproduction in whole or part is prohibited without
*                 written consent of the copyright owner. You may receive a free
*                 commercial license by contacting Biricha Digital. Do not
*                 remove or change this copyright notice.
*
*******************************************************************************/

#ifndef CSL_PWM_H
#define	CSL_PWM_H

/* Microcontroller MIPs (FCY) */
#define PWM_PS 1065LL   //it should ne 1040ps, but we measured it to be this

#define PWM_HI_RES

#ifdef PWM_HI_RES
#define PWM_PCLKDIV 0
#define PWM_DIV     1 // 900MHz / 1 = 900MHz
#else
#define PWM_PCLKDIV 4
#define PWM_DIV     16 // 900MHz / 16 = 60MHz
#endif


typedef enum PWM_Current_Limit_Mode //FCLCONx.FLTMOD<1:0>: Fault Mode for PWMx Generator # bits
{
    PWM_Cur_Limit_Mode_Disabled = 0, //Fault input is disabled
    PWM_Cur_Limit_Mode_Cycle    = 1  //The selected Fault source forces PWMxH, PWMxL pins to FLTDAT values (cycle)
}PWM_Current_Limit_Mode;

typedef enum PWM_Pin_Level
{
    PWM_Pin_Low = 0,
    PWM_Pin_High = 1
}PWM_Pin_Level;

typedef enum PWM_Pin_Output
{
    PWM_Pin_Disable,
    PWM_Pin_Active_High,
    PWM_Pin_Active_Low
}PWM_Pin_Output;

typedef enum PWM_Mode
{
    PWM_Mode_Complementary = 0, //the PWM output PWMxH, is the complement of the PWMxL output with deadtime
    PWM_Mode_Redundant = 1, //the PWM output PWMxH and PWMxL are the same
    PWM_Mode_Push_Pull = 2, //the PWM outputs are alternately available on the PWMxH and PWMxL pins
    PWM_Mode_Independent = 3 //the PWM outputs (PWMxH and PWMxL) can have different duty cycles and are phase shifted relative to each other
}PWM_Mode;

typedef enum PWM_Deadband
{
    PWM_Deadband_Pos = 0, //Positive dead time actively applied for all output modes
    PWM_Deadband_Neg = 1, //Negative dead time actively applied for all output modes
    PWM_Deadband_Disable = 2 //Dead time function is disabled
}PWM_Deadband;

#define PWM_freqToTicks(FreqHz) (1000000000000LL / (PWM_PS *PWM_DIV* FreqHz))   // PTPER = ((1 / 350kHz) / 1.04ns) = 2747, where 350kHz
#define PWM_nsToTicks(ns) ( (1000LL*(ns)) / (PWM_PS *PWM_DIV))   // PTPER = ((1 / 350kHz) / 1.04ns) = 2747, where 350kHz
#define PWM_blankingNsToTicks( ns ) ((1000LL*(ns))/(8320LL*PWM_DIV)) //steps of 8.32ns or 133.12ns

extern void PWM_init(uint16_t PeriodTicks);
extern void PWM_enable( bool Enable);

extern void PWM1_setOutput( PWM_Mode Mode, PWM_Pin_Output PwmH, PWM_Pin_Output PwmL);
extern void PWM1_config( void );
extern void PWM1_blanking( PWM_Current_Limit_Mode CurrentLimitMode, uint16_t Ticks, PWM_Pin_Level PwmH, PWM_Pin_Level PwmL);
extern void PWM1_interrupt( INT_Priority Priority );
extern void PWM1_deadtime( PWM_Deadband Mode, uint16_t Ticks );

#define PWM1_setDutyH( Ticks ) PDC1 = (Ticks)
#define PWM1_setDutyL( Ticks ) SDC1 = (Ticks)
#define PWM1_clrInt() IFS5bits.PWM1IF = 0      //Clear the interrupt flag
#define PWM1_setTrigger( Ticks ) TRIG1 = (Ticks)
#endif	/* CSL_PWM_H */

